package com.example.nto.service.impl;

import com.example.nto.entity.Employee;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.EmployeeService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Service
public class EmployeeServiceImpl implements EmployeeService {

    private final EmployeeRepository employeeRepository;
    private final BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
    private final CodeRepository codeRepository;


    public EmployeeServiceImpl(EmployeeRepository employeeRepository, CodeRepository codeRepository) {
        this.employeeRepository = employeeRepository;
        this.codeRepository = codeRepository;
    }

    @Override
    public List<Employee> findAll() {
        return employeeRepository.findAll();
    }

    @Override
    public Optional<Employee> findByLogin(String login) {
        return Optional.ofNullable(employeeRepository.findByLogin(login));
    }

    @Override
    public boolean validateCode(String login, long code) {
        return false;
    }

    @Override
    public boolean validateCode(long code) {
        return codeRepository.existsByValue(code);
    }

    @Override
    public boolean openDoor(String login, long code) {
        if (!validateCode(code)) {  // Проверяем, есть ли код в таблице `code`
            return false; // Код неверный
        }

        Optional<Employee> employeeOpt = findByLogin(login);
        if (employeeOpt.isPresent()) {
            Employee employee = employeeOpt.get();

            // Обновляем данные входа сотрудника
            employee.setLastVisit(LocalDateTime.now()); // Фиксируем время входа
            employee.setLastEntryCode(code); // Записываем использованный код
            employee.setEntryMethod("QR"); // Фиксируем метод входа

            // Сохраняем изменения
            employeeRepository.save(employee);

            return true; // Дверь открывается
        }
        return false;
    }



    public boolean authenticate(String login, String rawPassword) {
        Optional<Employee> employee = findByLogin(login);
        if (employee.isPresent() && !employee.get().isBlocked()) {
            return passwordEncoder.matches(rawPassword, employee.get().getPassword());
        }
        return false;
    }

    public void toggleBlockStatus(String login) {
        Optional<Employee> employee = findByLogin(login);
        employee.ifPresent(emp -> {
            emp.setStatus(emp.isBlocked() ? "works" : "blocked");
            employeeRepository.save(emp);
        });
    }


}